﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BookService
{
    public class Book : IComparable<Book>
    {
        public String title;
        public String publisher;
        public String year;
        public bool inStock;
        public int numInStock;

        private static Dictionary<Author, Book[]> booksByAuthor;

        static Book()
        {
            booksByAuthor = new Dictionary<Author, Book[]>();

            foreach(Author a in Author.getAuthors())
            {
                switch (a.lastName)
                {
                    case "DeLillo":
                        booksByAuthor[a] = new Book[]
                        {
                            new Book("White Noise", "Viking", "1985", true, 7),
                            new Book("Underworld", "Scribner", "1997", true, 15),
                            new Book("Cosmopolis", "Scribner", "2003", false, 0)
                        };
                        break;
                    case "Pynchon":
                        booksByAuthor[a] = new Book[]
                        {
                            new Book("The Crying of Lot 49", "Lippincot", "1966", true, 5),
                            new Book("Gravity's Rainbow", "Viking", "1973", true, 10),
                            new Book("Inherent Vice", "Penguin", "2009", true, 3)
                        };
                        break;
                    case "Stephenson":
                        booksByAuthor[a] = new Book[]
                        {
                            new Book("Snow Crash", "Bantam", "1992", true, 8),
                            new Book("Cryptonomicon", "Avon", "1999", true, 9),
                            new Book("Anathem", "William Morrow", "2008", true, 2)
                        };
                        break;
                }
            }
        }

        public Book(String title, String publisher, String year, bool inStock, int numInStock)
        {
            this.title = title;
            this.publisher = publisher;
            this.year = year;
            this.inStock = inStock;
            this.numInStock = numInStock;
        }

        public static Book[] getBooks()
        {
            List<Book> allBooks = new List<Book>();
            foreach (Author a in Author.getAuthors())
            {
                Book[] authorBooks = booksByAuthor[a];
                allBooks.AddRange(authorBooks);
            }

            // sort allBooks by title
            allBooks.Sort();

            return allBooks.ToArray();
        }

        public static Book[] getBooks(String firstName, String lastName)
        {
            Author a = new Author(firstName, lastName);
            Book[] result = booksByAuthor[a];

            // sort result by title
            Array.Sort(result);

            return result;
        }

        public int CompareTo(Book other)
        {
            return this.title.CompareTo(other.title);
        }

        public override string ToString()
        {
            return "Book: " + title;
        }
    }
}
